@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2022 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2022 Laszlo Molnar
+   Copyright (C) 1996-2023 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2023 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -38,35 +38,35 @@
 // core - NE
 **************************************************************************/
 
-__acc_static_forceinline unsigned get_ne16(const void *p) {
+static forceinline unsigned get_ne16(const void *p) {
     upx_uint16_t v = 0;
     upx_memcpy_inline(&v, p, sizeof(v));
     return v;
 }
 
-__acc_static_forceinline unsigned get_ne32(const void *p) {
+static forceinline unsigned get_ne32(const void *p) {
     upx_uint32_t v = 0;
     upx_memcpy_inline(&v, p, sizeof(v));
     return v;
 }
 
-__acc_static_forceinline upx_uint64_t get_ne64(const void *p) {
+static forceinline upx_uint64_t get_ne64(const void *p) {
     upx_uint64_t v = 0;
     upx_memcpy_inline(&v, p, sizeof(v));
     return v;
 }
 
-__acc_static_forceinline void set_ne16(void *p, unsigned vv) {
-    upx_uint16_t v = (upx_uint16_t)(vv & 0xffff);
+static forceinline void set_ne16(void *p, unsigned vv) {
+    upx_uint16_t v = (upx_uint16_t) (vv & 0xffff);
     upx_memcpy_inline(p, &v, sizeof(v));
 }
 
-__acc_static_forceinline void set_ne32(void *p, unsigned vv) {
+static forceinline void set_ne32(void *p, unsigned vv) {
     upx_uint32_t v = vv;
     upx_memcpy_inline(p, &v, sizeof(v));
 }
 
-__acc_static_forceinline void set_ne64(void *p, upx_uint64_t vv) {
+static forceinline void set_ne64(void *p, upx_uint64_t vv) {
     upx_uint64_t v = vv;
     upx_memcpy_inline(p, &v, sizeof(v));
 }
@@ -79,36 +79,31 @@ __acc_static_forceinline void set_ne64(void *p, upx_uint64_t vv) {
 
 ACC_COMPILE_TIME_ASSERT_HEADER(sizeof(long) == 4)
 
-__acc_static_forceinline unsigned bswap16(unsigned v) {
-    return (unsigned) _byteswap_ulong(v << 16);
-}
-__acc_static_forceinline unsigned bswap32(unsigned v) { return (unsigned) _byteswap_ulong(v); }
-__acc_static_forceinline upx_uint64_t bswap64(upx_uint64_t v) { return _byteswap_uint64(v); }
+// unfortunately *not* constexpr with MSVC
+static forceinline unsigned bswap16(unsigned v) { return (unsigned) _byteswap_ulong(v << 16); }
+static forceinline unsigned bswap32(unsigned v) { return (unsigned) _byteswap_ulong(v); }
+static forceinline upx_uint64_t bswap64(upx_uint64_t v) { return _byteswap_uint64(v); }
 
 #else
 
-__acc_static_forceinline constexpr unsigned bswap16(unsigned v) {
+static forceinline constexpr unsigned bswap16(unsigned v) {
     // return __builtin_bswap16((upx_uint16_t) (v & 0xffff));
     // return (unsigned) __builtin_bswap64((upx_uint64_t) v << 48);
     return __builtin_bswap32(v << 16);
 }
-__acc_static_forceinline constexpr unsigned bswap32(unsigned v) {
+static forceinline constexpr unsigned bswap32(unsigned v) {
     // return (unsigned) __builtin_bswap64((upx_uint64_t) v << 32);
     return __builtin_bswap32(v);
 }
-__acc_static_forceinline constexpr upx_uint64_t bswap64(upx_uint64_t v) {
-    return __builtin_bswap64(v);
-}
+static forceinline constexpr upx_uint64_t bswap64(upx_uint64_t v) { return __builtin_bswap64(v); }
 
 #endif
 
-__acc_static_forceinline constexpr unsigned no_bswap16(unsigned v) {
+static forceinline constexpr unsigned no_bswap16(unsigned v) {
     return v & 0xffff; // needed so that this is equivalent to bswap16() above
 }
-
-__acc_static_forceinline constexpr unsigned no_bswap32(unsigned v) { return v; }
-
-__acc_static_forceinline constexpr upx_uint64_t no_bswap64(upx_uint64_t v) { return v; }
+static forceinline constexpr unsigned no_bswap32(unsigned v) { return v; }
+static forceinline constexpr upx_uint64_t no_bswap64(upx_uint64_t v) { return v; }
 
 #if (ACC_ABI_BIG_ENDIAN)
 #define ne16_to_be16(v) no_bswap16(v)
@@ -175,24 +170,27 @@ inline unsigned get_le26(const void *p) { return get_le32(p) & 0x03ffffff; }
 
 inline void set_le26(void *p, unsigned v) {
     // preserve the top 6 bits
-    // set_le32(p, (get_le32(p) & 0xfc000000) | (v & 0x03ffffff));
+#if 0
+    set_le32(p, (get_le32(p) & 0xfc000000) | (v & 0x03ffffff));
+#else
     // optimized version, saving a runtime bswap32
     set_ne32(p, (get_ne32(p) & ne32_to_le32(0xfc000000)) |
                     (ne32_to_le32(v) & ne32_to_le32(0x03ffffff)));
+#endif
 }
 
 /*************************************************************************
 // get signed values
 **************************************************************************/
 
-__acc_static_forceinline int sign_extend(unsigned v, unsigned bits) {
+static forceinline int sign_extend(unsigned v, unsigned bits) {
     const unsigned sign_bit = 1u << (bits - 1);
     v &= sign_bit | (sign_bit - 1);
     v |= 0 - (v & sign_bit);
     return ACC_ICAST(int, v);
 }
 
-__acc_static_forceinline upx_int64_t sign_extend(upx_uint64_t v, unsigned bits) {
+static forceinline upx_int64_t sign_extend(upx_uint64_t v, unsigned bits) {
     const upx_uint64_t sign_bit = 1ull << (bits - 1);
     v &= sign_bit | (sign_bit - 1);
     v |= 0 - (v & sign_bit);
@@ -251,6 +249,7 @@ inline upx_int64_t get_le64_signed(const void *p) {
 **************************************************************************/
 
 struct alignas(1) BE16 {
+    typedef unsigned integral_conversion_type; // automatic conversion to unsigned
     unsigned char d[2];
 
     BE16 &operator=(unsigned v) {
@@ -300,6 +299,7 @@ struct alignas(1) BE16 {
 };
 
 struct alignas(1) BE32 {
+    typedef unsigned integral_conversion_type; // automatic conversion to unsigned
     unsigned char d[4];
 
     BE32 &operator=(unsigned v) {
@@ -349,6 +349,7 @@ struct alignas(1) BE32 {
 };
 
 struct alignas(1) BE64 {
+    typedef upx_uint64_t integral_conversion_type; // automatic conversion to upx_uint64_t
     unsigned char d[8];
 
     BE64 &operator=(upx_uint64_t v) {
@@ -398,6 +399,7 @@ struct alignas(1) BE64 {
 };
 
 struct alignas(1) LE16 {
+    typedef unsigned integral_conversion_type; // automatic conversion to unsigned
     unsigned char d[2];
 
     LE16 &operator=(unsigned v) {
@@ -447,6 +449,7 @@ struct alignas(1) LE16 {
 };
 
 struct alignas(1) LE32 {
+    typedef unsigned integral_conversion_type; // automatic conversion to unsigned
     unsigned char d[4];
 
     LE32 &operator=(unsigned v) {
@@ -496,6 +499,7 @@ struct alignas(1) LE32 {
 };
 
 struct alignas(1) LE64 {
+    typedef upx_uint64_t integral_conversion_type; // automatic conversion to upx_uint64_t
     unsigned char d[8];
 
     LE64 &operator=(upx_uint64_t v) {
@@ -567,7 +571,6 @@ template <class T>
 inline T *operator-(T *ptr, const BE16 &v) {
     return ptr - unsigned(v);
 }
-
 template <class T>
 inline T *operator+(T *ptr, const BE32 &v) {
     return ptr + unsigned(v);
@@ -576,13 +579,6 @@ template <class T>
 inline T *operator-(T *ptr, const BE32 &v) {
     return ptr - unsigned(v);
 }
-
-// these are not implemented on purpose and will cause link-time errors
-template <class T>
-T *operator+(T *ptr, const BE64 &v);
-template <class T>
-T *operator-(T *ptr, const BE64 &v);
-
 template <class T>
 inline T *operator+(T *ptr, const LE16 &v) {
     return ptr + unsigned(v);
@@ -591,7 +587,6 @@ template <class T>
 inline T *operator-(T *ptr, const LE16 &v) {
     return ptr - unsigned(v);
 }
-
 template <class T>
 inline T *operator+(T *ptr, const LE32 &v) {
     return ptr + unsigned(v);
@@ -601,11 +596,15 @@ inline T *operator-(T *ptr, const LE32 &v) {
     return ptr - unsigned(v);
 }
 
-// these are not implemented on purpose and will cause link-time errors
+// these are not implemented on purpose and will cause errors
 template <class T>
-T *operator+(T *ptr, const LE64 &v);
+T *operator+(T *ptr, const BE64 &v) DELETED_FUNCTION;
 template <class T>
-T *operator-(T *ptr, const LE64 &v);
+T *operator-(T *ptr, const BE64 &v) DELETED_FUNCTION;
+template <class T>
+T *operator+(T *ptr, const LE64 &v) DELETED_FUNCTION;
+template <class T>
+T *operator-(T *ptr, const LE64 &v) DELETED_FUNCTION;
 
 /*************************************************************************
 // global overloads
